
# q-expansion of newform 192.9.g.c, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 192
weight = 9
poly_data = [202500, 450, 451, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[101025, -451, 451, -1], 101475], [[32424, 0, 0, 48], 451], [[1800, 7208, -8, 8], 75]]

hecke_ring_character_values = [[127, [-1, 0, 0, 0]], [133, [1, 0, 0, 0]], [65, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, -27, 0, 0], [66, 0, 1, 0], [0, -772, 0, -1], [0, 6180, 0, -2], [-3658, 0, -30, 0], [83226, 0, -62, 0], [0, 6956, 0, 118], [0, -72576, 0, 126], [-585894, 0, -409, 0], [0, -597420, 0, 343], [-1078946, 0, 1836, 0], [2258874, 0, -882, 0], [0, -980892, 0, -1814], [0, -664152, 0, 194], [10546650, 0, -1569, 0], [0, 1739100, 0, -3112], [12037102, 0, 5712, 0], [0, -14479276, 0, 5592], [0, -16653264, 0, 5462], [-5303870, 0, 25944, 0], [0, 5791732, 0, 26507], [0, 16123836, 0, 32510], [-3168894, 0, -33220, 0], [65788450, 0, 59532, 0], [16085754, 0, -83649, 0], [0, -62492620, 0, -54543], [0, 25370556, 0, -116204], [-109492090, 0, -51234, 0], [-27806766, 0, -41972, 0], [0, -1398564, 0, 50629], [0, -159339948, 0, -9712], [-59813526, 0, 425642, 0], [0, -155071508, 0, -143120], [493895778, 0, 83129, 0], [0, -176205908, 0, 167943], [-193894658, 0, 63852, 0], [0, 49991500, 0, 65218], [0, 418865928, 0, 411352], [-1121012382, 0, -475783, 0], [0, -114097308, 0, -931576], [-529459354, 0, -1231842, 0], [0, 204700224, 0, -1107112], [-526215262, 0, -20760, 0], [-947216406, 0, 1267643, 0], [0, -647775676, 0, 3493], [0, 931846468, 0, -931576], [0, 729621612, 0, 1168105], [0, -777056484, 0, 554026], [1750756598, 0, 413202, 0], [252835170, 0, -2202292, 0], [0, 408072000, 0, -433372], [-2258575022, 0, -674100, 0], [0, 1457628276, 0, 62414], [2113671426, 0, -1536640, 0], [0, -448436952, 0, 1129824], [5870478234, 0, -3334721, 0], [0, 735444316, 0, -701991], [-8210700922, 0, -2434770, 0], [2093398770, 0, -4614944, 0], [0, -1586516868, 0, -1657184], [1678889946, 0, 151927, 0], [0, 2469854628, 0, -5468424], [0, -671361432, 0, -2863012], [-1651896718, 0, -1911528, 0], [3709028298, 0, 4843963, 0], [0, -7094909540, 0, -1525996], [-2647345550, 0, -14626536, 0], [0, -7735567788, 0, 2114790], [3210123598, 0, -5105904, 0], [9932863122, 0, -9503124, 0], [0, 7658035152, 0, -5330474], [0, -3283522396, 0, 915783], [-6050587730, 0, -4984968, 0], [0, -4627808668, 0, -1321810], [0, -3696168096, 0, -4718208], [15903394434, 0, -5218959, 0], [1663713358, 0, 21762456, 0], [23999160426, 0, -7913674, 0], [17860551250, 0, -8732700, 0], [0, 55124412, 0, 6104914], [12022215590, 0, 13803822, 0], [0, -8889213960, 0, -9409602], [-8258524846, 0, 19216920, 0], [0, 6098994180, 0, 16019841], [0, -32212701900, 0, 5059254], [47818793466, 0, 12698538, 0], [20154457138, 0, 27117420, 0], [31592055330, 0, -23972703, 0], [0, -11345315908, 0, -14462831], [0, 25504205964, 0, -17665310], [0, 4080254856, 0, -7197602], [0, -22205286532, 0, 9222919], [0, -13927732068, 0, 163396], [0, -15005990556, 0, -33431232], [0, 26284693056, 0, 21577198], [7915929642, 0, -31961517, 0], [-58403330118, 0, 27891750, 0], [0, -6769628524, 0, -66017010], [76651270118, 0, -19358010, 0], [0, 65482789660, 0, -223650], [-5204108334, 0, 135190253, 0], [0, 73585698876, 0, -23359938], [32777098554, 0, 101758174, 0], [0, -63180276340, 0, -30136108], [105755057810, 0, 29433432, 0], [0, 100087357020, 0, 10628188], [46745880018, 0, 96260876, 0], [0, -26460093456, 0, -13452270], [8276456642, 0, -135528864, 0], [0, 15374498972, 0, 21652157], [75816540910, 0, 79964208, 0], [65698709586, 0, -103274104, 0], [0, -37142676340, 0, 21763896], [0, -6895702196, 0, 4802119], [-168791810502, 0, -47386942, 0], [0, -59662455588, 0, -672426], [0, 12511630896, 0, -39259298], [-8939997774, 0, -159515547, 0], [0, -30688811436, 0, -31419716], [91232194846, 0, 197314332, 0], [-134231003854, 0, 116936304, 0], [-231404343246, 0, -146654859, 0], [0, -110821684908, 0, 128299690], [0, -78215695236, 0, -30874222], [236202291498, 0, 84823579, 0], [214932184598, 0, 83425650, 0], [0, -46239009432, 0, -137420350], [0, -55542155796, 0, -179750745], [-257053948714, 0, 139247946, 0], [0, 197184322404, 0, -126312652], [0, 12093695640, 0, 119104116], [0, 161937137524, 0, -48676493], [-83529689482, 0, -46279134, 0], [-214540233078, 0, -175063222, 0], [516524445122, 0, 19295664, 0], [-526126305270, 0, 135325187, 0], [0, -121624116484, 0, 82519554], [-318740746926, 0, -258172387, 0], [-38307157590, 0, 557767034, 0], [0, 180716205060, 0, 104954794], [420336855210, 0, 101501227, 0], [0, 69134117980, 0, 303771635], [0, 63340827084, 0, -12885828], [114534863846, 0, -260286450, 0], [0, -213003247392, 0, 91324410], [-2701249634, 0, -721851084, 0], [-34337827398, 0, -252947842, 0], [0, -168711856700, 0, 28575478], [0, 217424575968, 0, -126814520], [-73049890514, 0, -262652616, 0], [-880193296926, 0, 42750216, 0], [0, 141846643036, 0, 512657306], [0, -344294675304, 0, -233723824], [0, 176455556084, 0, -265646402], [0, 8371801920, 0, 154085612], [0, -217088159364, 0, -29775597], [173252525898, 0, 947190430, 0], [267197927570, 0, 1058277168, 0], [580109745282, 0, -554408983, 0], [0, 76178051748, 0, 701085096], [-117730699014, 0, -271943914, 0], [0, 447825893044, 0, 74810073], [0, 575939863380, 0, 46662338], [1393625182986, 0, 3101022, 0], [0, -216433279848, 0, -489450200], [0, 253158524612, 0, 686068635], [-627473070194, 0, -606086616, 0]]
